import 'package:flutter/widgets.dart';
import 'package:get/get.dart';

/**
 * Created by bayu404.dart@gmail.com
 * Copyright (c) 2021 . All rights reserved.
 */

abstract class DeasytStateWidget<T extends DeasytStateController>
    extends StatelessWidget {

  const DeasytStateWidget(
      this.builder, {
        this.tag,
        this.permanent = false,
        this.args = const <String, dynamic>{},
        Key? key,
      }) : super(key: key);

  const DeasytStateWidget.lazyAdjust({
    String? tag,
    bool markAsPermanent = false,
    Map<String, dynamic> args = const <String, dynamic>{},
    Key? key,
  }) : this(null, tag: tag, permanent: markAsPermanent, args: args, key: key);

  final InstanceBuilderCallback<T>? builder;

  final String? tag;
  final bool permanent;
  final Map<String, dynamic> args;

  T get controller => GetInstance().find<T>(tag: tag);

  Widget buildWidget(BuildContext context);

  @override
  Widget build(BuildContext context) {
    return Stack(
      fit: StackFit.passthrough,
      children: [
        Wrap(
          children: [
            _DeasytStateControllerManager<T>(
              builder,
              tag: tag,
              permanent: permanent,
              args: args,
            ),
          ],
        ),
        buildWidget(context),
      ],
    );
  }
}

class _DeasytStateControllerManager<T extends DeasytStateController> extends StatefulWidget {
  _DeasytStateControllerManager(
      InstanceBuilderCallback<T>? builder, {
        this.tag,
        this.permanent = false,
        Map<String, dynamic> args = const <String, dynamic>{},
        Key? key,
      }) : super(key: key) {
    final inst = GetInstance();
    if (builder != null && !inst.isRegistered<T>(tag: tag)) {
      inst.put(builder(), tag: tag, permanent: permanent);
    }
    final c = inst.find<T>(tag: tag);
    c.args = args;
  }

  final String? tag;
  final bool permanent;

  @override
  _DeasytStateControllerManagerState<T> createState() =>
      _DeasytStateControllerManagerState<T>();
}

class _DeasytStateControllerManagerState<T extends DeasytStateController>
    extends State<_DeasytStateControllerManager<T>> {
  @override
  initState() {
    super.initState();
    final wc = GetInstance().find<T>(tag: widget.tag);
    wc.onWidgetInitState();
  }

  @override
  void dispose() {
    final inst = GetInstance();
    if (inst.isRegistered<T>(tag: widget.tag)) {
      final wc = inst.find<T>(tag: widget.tag);
      wc.onWidgetDisposed();
      if (!widget.permanent) {
        Get.delete<T>(tag: widget.tag);
      }
    }
    super.dispose();
  }

  @override
  Widget build(BuildContext context) => Container();
}

abstract class DeasytStateController extends GetxController {
  final _args = <String, dynamic>{};

  Map<String, dynamic> get args => _args;

  set args(Map<String, dynamic> value) => _args.assignAll(value);

  void onWidgetInitState() {}

  void onWidgetDisposed() {}
}
